@@ -33,6 +33,7 @@
 
 static int dac_volumes;
 static int dac_max = 127;
+static int adc_max = 127;
 static int adc_volumes;
 static int ipga_volumes;
 static int dac_senses;
@@ -58,6 +59,11 @@ int envy_adc_volumes(void)
 	return adc_volumes;
 }
 
+int envy_adc_max(void)
+{
+	return adc_max;
+}
+
 int envy_ipga_volumes(void)
 {
 	return ipga_volumes;
@@ -138,8 +144,9 @@ void adc_volume_update(int idx)
 		g_print("Unable to read ipga volume: %s\n", snd_strerror(err));
 		return;
 	}
-	gtk_adjustment_set_value(GTK_ADJUSTMENT(av_ipga_volume_adj[idx]),
-				 -0);
+	if (ipga_volumes > 0)
+		gtk_adjustment_set_value(GTK_ADJUSTMENT(av_ipga_volume_adj[idx]),
+					 -0);
 }
 
 void ipga_volume_update(int idx)
@@ -165,7 +172,7 @@ void ipga_volume_update(int idx)
 	// set ADC volume to max if IPGA volume greater 0
 	if (ipga_vol)
 		gtk_adjustment_set_value(GTK_ADJUSTMENT(av_adc_volume_adj[idx]),
-					 -127);
+					 -adc_max);
 }
 
 void dac_sense_update(int idx)
@@ -208,7 +215,7 @@ void adc_sense_update(int idx)
 
 void dac_volume_adjust(GtkAdjustment *adj, gpointer data)
 {
-	int idx = (int)data;
+	int idx = (int)(long)data;
 	snd_ctl_elem_value_t *val;
 	int err, ival = -(int)adj->value;
 	char text[16];
@@ -226,7 +233,7 @@ void dac_volume_adjust(GtkAdjustment *adj, gpointer data)
 
 void adc_volume_adjust(GtkAdjustment *adj, gpointer data)
 {
-	int idx = (int)data;
+	int idx = (int)(long)data;
 	snd_ctl_elem_value_t *val;
 	int err, ival = -(int)adj->value;
 	char text[16];
@@ -244,7 +251,7 @@ void adc_volume_adjust(GtkAdjustment *adj, gpointer data)
 
 void ipga_volume_adjust(GtkAdjustment *adj, gpointer data)
 {
-	int idx = (int)data;
+	int idx = (int)(long)data;
 	snd_ctl_elem_value_t *val;
 	int err, ival = -(int)adj->value;
 	char text[16];
@@ -342,6 +349,7 @@ void analog_volume_init(void)
 		snd_ctl_elem_info_set_index(info, i);
 		if (snd_ctl_elem_info(ctl, info) < 0)
 			break;
+		adc_max = snd_ctl_elem_info_get_max(info);
 	}
 	if (i < input_channels - 1)
 		adc_volumes = i;
@@ -386,15 +394,15 @@ void analog_volume_postinit(void)
 
 	for (i = 0; i < dac_volumes; i++) {
 		dac_volume_update(i);
-		dac_volume_adjust((GtkAdjustment *)av_dac_volume_adj[i], (gpointer)i);
+		dac_volume_adjust((GtkAdjustment *)av_dac_volume_adj[i], (gpointer)(long)i);
 	}
 	for (i = 0; i < adc_volumes; i++) {
 		adc_volume_update(i);
-		adc_volume_adjust((GtkAdjustment *)av_adc_volume_adj[i], (gpointer)i);
+		adc_volume_adjust((GtkAdjustment *)av_adc_volume_adj[i], (gpointer)(long)i);
 	}
 	for (i = 0; i < ipga_volumes; i++) {
 		ipga_volume_update(i);
-		ipga_volume_adjust((GtkAdjustment *)av_ipga_volume_adj[i], (gpointer)i);
+		ipga_volume_adjust((GtkAdjustment *)av_ipga_volume_adj[i], (gpointer)(long)i);
 	}
 	for (i = 0; i < dac_senses; i++)
 		dac_sense_update(i);
